@@ -9,7 +9,7 @@
 # Author:      Robin Dunn
 #
 # Created:     3-Dec-2010
-# Copyright:   (c) 2010-2018 by Total Control Software
+# Copyright:   (c) 2010-2020 by Total Control Software
 # License:     wxWindows License
 #----------------------------------------------------------------------
 
@@ -27,12 +27,24 @@ import tarfile
 import tempfile
 import datetime
 
+try:
+    import pathlib
+except ImportError:
+    import buildtools.backports.pathlib2 as pathlib
+
+try:
+    from shutil import which
+except ImportError:
+    from buildtools.backports.shutil_which import which
+
+
 from distutils.dep_util import newer, newer_group
 from buildtools.config  import Config, msg, opj, posixjoin, loadETG, etg2sip, findCmd, \
                                phoenixDir, wxDir, copyIfNewer, copyFile, \
                                macSetLoaderNames, \
                                getVcsRev, runcmd, textfile_open, getSipFiles, \
-                               getVisCVersion, getToolsPlatformName, updateLicenseFiles
+                               getVisCVersion, getToolsPlatformName, updateLicenseFiles, \
+                               TemporaryDirectory
 
 import buildtools.version as version
 
@@ -70,21 +82,21 @@ baseName = version.PROJECT_NAME
 eggInfoName = baseName + '.egg-info'
 defaultMask='%s-%s*' % (baseName, version.VER_MAJOR)
 
-pyICON = 'docs/sphinx/_static/images/sphinxdocs/phoenix_title.png'
-wxICON = 'docs/sphinx/_static/images/sphinxdocs/mondrian.png'
+pyICON = 'packaging/docset/Vippi-blocks-icon-32.png'
+wxICON = 'packaging/docset/mondrian.png'
 
 # Some tools will be downloaded for the builds. These are the versions and
 # MD5s of the tool binaries currently in use.
-sipCurrentVersion = '4.19.7'
+sipCurrentVersion = '4.19.19'
 sipMD5 = {
-    'darwin'   : 'd30ca1ffb09c0dbb6326e99d012c55b8',
-    'win32'    : 'dbb882f4f95b1a7419a436280407899c',
-    'linux32'  : '56a763acdf7c0b5725b31a71a9a56160',
-    'linux64'  : 'b349127a4d46452936e4181d96b12c2d',
+    'darwin'   : '3309e69f2341f11bac263d7faea19616',
+    'win32'    : 'fd8be5c5fa369a848e907acad6879846',
+    'linux32'  : 'd0271b47695e74483f0c0d54f24819d5',
+    'linux64'  : '355410a19846768cdeb53efe249aabdb',
 }
 
-wafCurrentVersion = '2.0.7'
-wafMD5 = '48ac1250bcccd0674cf461937875ce9a'
+wafCurrentVersion = '2.0.19'
+wafMD5 = 'ac362b60111a59ab2df63513018d5ad8'
 
 doxygenCurrentVersion = '1.8.8'
 doxygenMD5 = {
@@ -117,8 +129,6 @@ Usage: ./build.py [command(s)] [options]
       sip           Run sip to generate the C++ wrapper source
 
       wxlib         Build the Sphinx input files for wx.lib
-      wxpy          Build the Sphinx input files for wx.py
-      wxtools       Build the Sphinx input files for wx.tools
       sphinx        Run the documentation building process using Sphinx
 
       docset        Build Dash or Zeal compatible docsets
@@ -257,7 +267,7 @@ def setPythonVersion(args):
             #
             TOOLS = os.environ.get('TOOLS')
             if 'cygdrive' in TOOLS:
-                TOOLS = runcmd(getCygwinPath()+'/bin/cygpath -w '+TOOLS, True, False)
+                TOOLS = bash2dosPath(TOOLS)
             use64flag = '--x64' in args
             if use64flag:
                 args.remove('--x64')
@@ -317,7 +327,7 @@ def setDevModeOptions(args):
     myDevModeOptions = [
             #'--build_dir=../bld',
             #'--prefix=/opt/wx/2.9',
-            '--jobs=6', #  % numCPUs(),
+            '--jobs=8', #  % numCPUs(),
 
             # These will be ignored on the other platforms so it is okay to
             # include them unconditionally
@@ -326,6 +336,7 @@ def setDevModeOptions(args):
             #'--osx_carbon',
             #'--mac_arch=i386',
             #'--mac_arch=i386,x86_64',
+            '--no_allmo',
             ]
     if not isWindows:
         myDevModeOptions.append('--debug')
@@ -435,7 +446,10 @@ def makeOptionParser():
         ("jom",            (False, "Use jom instead of nmake for the wxMSW build")),
         ("pytest_timeout", ("0",   "Timeout, in seconds, for stopping stuck test cases. (Currently not working as expected, so disabled by default.)")),
         ("pytest_jobs",    ("",    "Number of parallel processes py.test should run")),
-        ("vagrant_vms",    ("all", "Comma separated list of VM names to use for the build_vagrant command. Defaults to \"all\"")),
+        ("docker_img",     ("all", "Comma separated list of image tags to use for the build_docker command. Defaults to \"all\"")),
+        ("dump_waf_log",   (False, "If the waf build tool fails then using this option will cause waf's configure log to be printed")),
+        ("regenerate_sysconfig", (False, "Waf uses Python's sysconfig and related tools to configure the build. In some cases that info can be incorrect, so this option regenerates it. Must have write access to Python's lib folder.")),
+        ("no_allmo",       (False, "Skip regenerating the wxWidgets message catalogs")),
         ]
 
     parser = optparse.OptionParser("build options:")
@@ -562,7 +576,8 @@ def getTool(cmdName, version, MD5, envVar, platformBinary, linuxBits=False):
             # now check the MD5 if not in dev mode and it's set to None
             if not (devMode and md5 is None):
                 m = hashlib.md5()
-                m.update(open(cmd, 'rb').read())
+                with open(cmd, 'rb') as fid:
+                    m.update(fid.read())
                 if m.hexdigest() != md5:
                     _error_msg('MD5 mismatch, got "%s"\n       '
                                'expected          "%s"' % (m.hexdigest(), md5))
@@ -580,7 +595,7 @@ def getTool(cmdName, version, MD5, envVar, platformBinary, linuxBits=False):
 
             try:
                 p = subprocess.Popen([cmd, '--help'], stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=os.environ)
-                p.wait()
+                p.communicate()
             except OSError as e:
                 _error_msg('Could not execute %s, got "%s"' % (cmd, e))
                 sys.exit(1)
@@ -667,6 +682,7 @@ def uploadPackage(fileName, options, mask=defaultMask, keep=75):
     space is not overly consumed. It is assumed that if the files are in
     sorted order then the end of the list will be the newest files.
     """
+    fileName = os.path.relpath(fileName)
     fileName = fileName.replace('\\', '/')
     msg("Uploading %s..." % fileName)
 
@@ -686,8 +702,8 @@ def uploadPackage(fileName, options, mask=defaultMask, keep=75):
     cmd = 'scp {} {}:{}'.format(fileName, host, uploadDir)
     runcmd(cmd)
 
-    # Make sure it is readable by all
-    cmd = 'ssh {} "cd {}; chmod a+r {}"'.format(host, uploadDir, os.path.basename(fileName))
+    # Make sure it is readable by all, and writable by rbot
+    cmd = 'ssh {} "cd {}; chmod 644 {}"'.format(host, uploadDir, os.path.basename(fileName))
     runcmd(cmd)
 
     if not options.release:
@@ -747,7 +763,7 @@ def checkCompiler(quiet=False):
         # Make sure that the compiler that Python wants to use can be found.
         # It will terminate if the compiler is not found or other exceptions
         # are raised.
-        cmd = "import distutils.msvc9compiler as msvc; " \
+        cmd = "import setuptools, distutils.msvc9compiler as msvc; " \
               "mc = msvc.MSVCCompiler(); " \
               "mc.initialize(); " \
               "print(mc.cc)"
@@ -758,7 +774,7 @@ def checkCompiler(quiet=False):
         # Now get the environment variables which that compiler needs from
         # its vcvarsall.bat command and load them into this process's
         # environment.
-        cmd = "import distutils.msvc9compiler as msvc; " \
+        cmd = "import setuptools, distutils.msvc9compiler as msvc; " \
               "arch = msvc.PLAT_TO_VCVARS[msvc.get_platform()]; " \
               "env = msvc.query_vcvarsall(msvc.VERSION, arch); " \
               "print(env)"
@@ -784,7 +800,7 @@ def checkCompiler(quiet=False):
     # we'll need to make this a little smarter about what flag (if any)
     # needs to be used.
     #
-    # NOTE 2: SIP chenged its output such that this doesn't appear to be
+    # NOTE 2: SIP changed its output such that this doesn't appear to be
     # needed anymore, but we'll leave the code in place to make it easy to
     # turn it back on again if/when needed.
     if False and not isWindows and not isDarwin:
@@ -805,22 +821,109 @@ def getWafBuildBase():
     return base
 
 
-def getCygwinPath():
+def getBashPath():
+    """Check if there is a bash.exe on the PATH"""
+    bash = which('bash.exe')
+    return bash
+
+
+def dos2bashPath(path):
     """
-    Try to locate the path where cygwin is installed.
-
-    If CYGWIN_BASE is set in the environment then use that. Otherwise look in
-    default install locations.
+    Convert an absolute dos-style path to one bash.exe can understand.
     """
-    if os.environ.get('CYGWIN_BASE'):
-        return os.environ.get('CYGWIN_BASE')
+    path = path.replace('\\', '/')
+    cygpath = which('cygpath')
+    wsl = which('wsl')
 
-    for path in ['c:/cygwin', 'c:/cygwin64']:
-        if os.path.isdir(path):
-            return path
+    # If we have cygwin then we can use cygpath to convert the path.
+    # Note that MSYS2 (and Git Bash) now also have cygpath so this should
+    # work there too.
+    if cygpath:
+        path = runcmd('"{}" -u "{}"'.format(cygpath, path), getOutput=True, echoCmd=False)
+        return path
+    elif wsl:
+        # Are we using Windows System for Linux? (untested)
+        path = runcmd('"{}" wslpath -a -u "{}"'.format(wsl, path), getOutput=True, echoCmd=False)
+        return path
+    else:
+        # Otherwise, do a simple translate and hope for the best?
+        # c:/foo --> /c/foo
+        # TODO: Check this!!
+        drive, rest = os.path.splitdrive(path)
+        path = '/{}/{}'.format(drive[0], rest)
+        return path
 
-    return None
 
+def bash2dosPath(path):
+    """
+    Convert an absolute unix-style path to one Windows can understand.
+    """
+    cygpath = which('cygpath')
+    wsl = which('wsl')
+
+    # If we have cygwin then we can use cygpath to convert the path.
+    # Note that MSYS2 (and Git Bash) now also have cygpath so this should
+    # work there too.
+    if cygpath:
+        path = runcmd('"{}" -w "{}"'.format(cygpath, path), getOutput=True, echoCmd=False)
+        return path
+    elif wsl:
+        # Are we using Windows System for Linux? (untested)
+        path = runcmd('"{}" wslpath -a -w "{}"'.format(wsl, path), getOutput=True, echoCmd=False)
+        return path
+    else:
+        # Otherwise, do a simple translate and hope for the best?
+        # /c/foo --> c:/foo
+        # There's also paths like /cygdrive/c/foo or /mnt/c/foo, but in those
+        # cases cygpath or wsl should be available.
+        components = path.split('/')
+        assert components[0] == '' and len(components[1]) == 1, "Expecting a path like /c/foo"
+        path = components[1] + ':/' + '/'.join(components[2:])
+        return path
+
+
+def do_regenerate_sysconfig():
+    """
+    If a Python environment has been relocated to a new folder then it's
+    possible that the sysconfig can still be using paths for the original
+    location. Since wxPython's build uses WAF, which uses the sysconfig (via
+    python-config, distutils.sysconfig, etc.) then we need to ensure that these
+    paths match the current environment.
+
+    TODO: Can this be done in a way that doesn't require overwriting a file in
+    the environment?
+    """
+    with TemporaryDirectory() as td:
+        pwd = pushDir(td)
+
+        # generate a new sysconfig data file
+        cmd = [PYTHON, '-m', 'sysconfig', '--generate-posix-vars']
+        runcmd(cmd)
+
+        # On success the new data module will have been written to a subfolder
+        # of the current folder, which is recorded in ./pybuilddir.txt
+        with open('pybuilddir.txt', 'r') as fp:
+            pybd = fp.read()
+
+        # grab the file in that folder and copy it into the Python lib
+        p = opj(td, pybd, '*')
+        datafile = glob.glob(opj(td, pybd, '*'))[0]
+        cmd = [PYTHON, '-c', 'import sysconfig; print(sysconfig.get_path("stdlib"))']
+        stdlib = runcmd(cmd, getOutput=True)
+        shutil.copy(datafile, stdlib)
+
+        del pwd
+
+
+def _setTarItemPerms(tarinfo):
+    """
+    Used to set permissions of the files and fodlers in the source tarball
+    """
+    if tarinfo.isdir():
+        tarinfo.mode = 0o755
+    else:
+        tarinfo.mode |= 0o644
+    return tarinfo
 
 
 #---------------------------------------------------------------------------
@@ -833,14 +936,18 @@ def _doDox(arg):
     doxCmd = os.path.abspath(doxCmd)
 
     if isWindows:
-        cygwin_path = getCygwinPath()
-        doxCmd = doxCmd.replace('\\', '/')
-        doxCmd = runcmd(cygwin_path+'/bin/cygpath -u '+doxCmd, True, False)
+        bash = getBashPath()
+        if not bash:
+            raise RuntimeError("ERROR: Unable to find bash.exe, needed for running regen.sh")
+
+        doxCmd = dos2bashPath(doxCmd)
+        print(doxCmd)
         os.environ['DOXYGEN'] = doxCmd
         os.environ['WX_SKIP_DOXYGEN_VERSION_CHECK'] = '1'
+
         d = posixjoin(wxDir(), 'docs/doxygen')
         d = d.replace('\\', '/')
-        cmd = '%s/bin/bash.exe -l -c "cd %s && ./regen.sh %s"' % (cygwin_path, d, arg)
+        cmd = '"{}" -l -c "cd {} && ./regen.sh {}"'.format(bash, d, arg)
     else:
         os.environ['DOXYGEN'] = doxCmd
         os.environ['WX_SKIP_DOXYGEN_VERSION_CHECK'] = '1'
@@ -860,8 +967,6 @@ def cmd_doxhtml(options, args):
     _doDox('chm')
 
 
-# NOTE: Because of the use of defaults and plutil the following docset
-# commands will only work correctly on OSX.
 def cmd_docset_wx(options, args):
     cmdTimer = CommandTimer('docset_wx')
     cfg = Config()
@@ -880,32 +985,71 @@ def cmd_docset_wx(options, args):
         shutil.rmtree(destname)
     shutil.move(srcname, destname)
     shutil.copyfile(wxICON, posixjoin(destname, 'icon.png'))
-    runcmd('plutil -convert xml1 {}/Contents/Info.plist'.format(destname))
 
 
 def cmd_docset_py(options, args):
     cmdTimer = CommandTimer('docset_py')
+    cfg = Config(noWxConfig=True)
     if not os.path.isdir('docs/html'):
-        msg('ERROR: No docs/html, has the sphinx build command been run?')
+        msg('ERROR: No docs/html dir found, has the sphinx build command been run?')
         sys.exit(1)
 
     # clear out any old docset build
-    name = 'wxPython-{}'.format(version3)
+    name = 'wxPython-{}'.format(cfg.VERSION)
     docset = posixjoin('dist', '{}.docset'.format(name))
     if os.path.isdir(docset):
         shutil.rmtree(docset)
 
     # run the docset generator
     VERBOSE = '--verbose' if options.verbose else ''
-    runcmd('doc2dash {} --name "{}" --icon {} --destination dist docs/html'
-           .format(VERBOSE, name, pyICON))
+    URL = 'https://docs.wxpython.org/' if options.release else 'https://wxpython.org/Phoenix/docs/html/'
 
-    # update its Info.plist file
-    docset = os.path.abspath(docset)
-    runcmd('defaults write {}/Contents/Info isJavaScriptEnabled true'.format(docset))
-    runcmd('defaults write {}/Contents/Info dashIndexFilePath index.html'.format(docset))
-    runcmd('defaults write {}/Contents/Info DocSetPlatformFamily wxpy'.format(docset))
-    runcmd('plutil -convert xml1 {}/Contents/Info.plist'.format(docset))
+    cmd = [PYTHON, '-m doc2dash', VERBOSE,
+           '--name', name, '--icon', pyICON,
+           '--index-page index.html', '--enable-js',
+           '--online-redirect-url', URL,
+           '--destination dist docs/html']
+    runcmd(' '.join(cmd))
+
+    # Remove the sidebar from the pages in the docset
+    msg('Removing sidebar from docset pages...')
+    _removeSidebar(opj('dist', name+'.docset', 'Contents', 'Resources', 'Documents'))
+
+    # build the tarball
+    msg('Archiving Phoenix docset...')
+    rootname = "wxPython-docset-{}".format(cfg.VERSION)
+    tarfilename = posixjoin(phoenixDir(), 'dist', '{}.tar.gz'.format(rootname))
+    if os.path.exists(tarfilename):
+        os.remove(tarfilename)
+    with tarfile.open(name=tarfilename, mode="w:gz") as tarball:
+        tarball.add(opj('dist', name+'.docset'), name+'.docset', filter=_setTarItemPerms)
+
+    if options.upload:
+        uploadPackage(tarfilename, options)
+
+    msg("Docset file built at %s" % tarfilename)
+
+
+
+def _removeSidebar(path):
+    """
+    Remove the sidebar <div> from the pages going into the docset
+    """
+    from bs4 import BeautifulSoup
+    for filename in glob.glob(opj(path, '*.html')):
+        with textfile_open(filename, 'rt') as f:
+            text = f.read()
+        text = text.replace('<script src="_static/javascript/sidebar.js" type="text/javascript"></script>', '')
+        soup = BeautifulSoup(text, 'html.parser')
+        tag = soup.find('div', 'sphinxsidebar')
+        if tag:
+            tag.extract()
+        tag = soup.find('div', 'document')
+        if tag:
+            tag.attrs['class'] = ['document-no-sidebar']
+        text = unicode(soup) if PY2 else str(soup)
+        with textfile_open(filename, 'wt') as f:
+            f.write(text)
 
 
 def cmd_docset(options, args):
@@ -986,7 +1130,7 @@ def cmd_sphinx(options, args):
     pwd2 = pushDir(sphinxDir)
     buildDir = os.path.join(sphinxDir, 'build')
     htmlDir = os.path.join(phoenixDir(), 'docs', 'html')
-    runcmd('sphinx-build -b html -d %s/doctrees . %s' % (buildDir, htmlDir))
+    runcmd('{} -m sphinx -b html -d {}/doctrees . {}'.format(PYTHON, buildDir, htmlDir))
     del pwd2
 
     msg('Postprocessing sphinx output...')
@@ -996,10 +1140,10 @@ def cmd_sphinx(options, args):
 def cmd_wxlib(options, args):
     from sphinxtools.modulehunter import ModuleHunter
 
-    cmdTimer = CommandTimer('wx.lib')
+    cmdTimer = CommandTimer('wxlib')
     pwd = pushDir(phoenixDir())
 
-    for wx_pkg in ['lib', 'py', 'tools']:
+    for wx_pkg in ['lib', 'py', 'svg', 'tools']:
         libDir = os.path.join(phoenixDir(), 'wx', wx_pkg)
 
         if not os.path.isdir(libDir):
@@ -1028,7 +1172,7 @@ def cmd_bdist_docs(options, args):
 
     msg("Archiving wxPython Phoenix documentation...")
     rootname = "%s-docs-%s" % (baseName, cfg.VERSION)
-    tarfilename = "dist/%s.tar.gz" % rootname
+    tarfilename = posixjoin(phoenixDir(), 'dist', '%s.tar.gz' % rootname)
 
     if not os.path.exists('dist'):
         os.makedirs('dist')
@@ -1037,7 +1181,7 @@ def cmd_bdist_docs(options, args):
 
     with tarfile.open(name=tarfilename, mode="w:gz") as tarball:
         tarball.add('docs/html', os.path.join(rootname, 'docs/html'),
-                    filter=lambda info: None if '.svn' in info.name else info)
+                    filter=_setTarItemPerms)
 
     if options.upload:
         uploadPackage(tarfilename, options, keep=5,
@@ -1107,10 +1251,14 @@ def cmd_sip(options, args):
         if not newer_group(sipFiles, sbf) and os.path.exists(pycode):
             continue
 
+        # leave this turned off for now...
+        # typehint = '-y {}'.format(posixjoin(cfg.PKGDIR, base[1:]) + '.pyi')
+        typehint = ''
+
         pycode = '-X pycode'+base+':'+pycode
         sip = getSipCmd()
-        cmd = '%s %s -c %s -b %s %s %s'  % \
-            (sip, cfg.SIPOPTS, tmpdir, sbf, pycode, src_name)
+        cmd = '%s %s -c %s -b %s %s %s %s'  % \
+            (sip, cfg.SIPOPTS, tmpdir, sbf, pycode, typehint, src_name)
         runcmd(cmd)
 
         classesNeedingClassInfo = { 'sip_corewxTreeCtrl.cpp' : 'wxTreeCtrl', }
@@ -1156,9 +1304,8 @@ def cmd_sip(options, args):
             if not os.path.exists(dest):
                 msg('%s is a new file, copying...' % os.path.basename(src))
                 srcTxt = processSrc(src, options.keep_hash_lines)
-                f = textfile_open(dest, 'wt')
-                f.write(srcTxt)
-                f.close()
+                with textfile_open(dest, 'wt') as f:
+                    f.write(srcTxt)
                 continue
 
             srcTxt = processSrc(src, options.keep_hash_lines)
@@ -1169,20 +1316,20 @@ def cmd_sip(options, args):
                 pass
             else:
                 msg('%s is changed, copying...' % os.path.basename(src))
-                f = textfile_open(dest, 'wt')
-                f.write(srcTxt)
-                f.close()
+                with textfile_open(dest, 'wt') as f:
+                    f.write(srcTxt)
 
         # Remove tmpdir and its contents
         shutil.rmtree(tmpdir)
 
 
-
 def cmd_touch(options, args):
     cmdTimer = CommandTimer('touch')
     pwd = pushDir(phoenixDir())
-    runcmd('touch etg/*.py')
-
+    etg = pathlib.Path('etg')
+    for item in etg.glob('*.py'):
+        item.touch()
+    cmd_touch_others(options, args)
 
 
 def cmd_test(options, args, tests=None):
@@ -1194,7 +1341,7 @@ def cmd_test(options, args, tests=None):
     # -n is the number of processes to run in parallel
     # --timeout will kill the test process if it gets stuck
     jobs = '-n{}'.format(options.pytest_jobs) if options.pytest_jobs else ''
-    boxed = '--boxed' if not isWindows else ''
+    boxed = '--forked' if not isWindows else ''
     sec = options.pytest_timeout
     timeout = '--timeout={}'.format(sec) if sec and sec != "0" else ''
     cmd = '"{}" -m pytest {} {} {} {} {} '.format(
@@ -1245,6 +1392,9 @@ def cmd_build_wx(options, args):
         if options.jom:
             build_options.append('--jom')
 
+        if PY2:
+            build_options.append('--no_dpi_aware')
+
     else:
         # Platform is something other than MSW
         if options.osx_carbon:
@@ -1332,16 +1482,17 @@ def cmd_build_wx(options, args):
         traceback.print_exc()
         sys.exit(1)
 
-    # Build the wx message catalogs, but first check that there is a msgfmt
-    # command available
-    if findCmd('msgfmt'):
-        locale_pwd = pushDir(posixjoin(wxDir(), 'locale'))
-        print('Building message catalogs in ' + os.getcwd())
-        runcmd('make allmo')
-        del locale_pwd
-    else:
-        print("WARNING: msgfmt command not found, message catalogs not rebulit.\n"
-              "         Please install gettext and associated tools.")
+    if not options.no_allmo:
+        # Build the wx message catalogs, but first check that there is a msgfmt
+        # command available
+        if findCmd('msgfmt') and findCmd('make'):
+            locale_pwd = pushDir(posixjoin(wxDir(), 'locale'))
+            print('Building message catalogs in ' + os.getcwd())
+            runcmd('make allmo')
+            del locale_pwd
+        else:
+            print("WARNING: msgfmt and/or make commands not found, message catalogs not \n"
+                "         rebuilt. Please install gettext and associated tools.")
 
 
 
@@ -1370,8 +1521,10 @@ def copyWxDlls(options):
             cairo_root = os.path.join(phoenixDir(), 'packaging', 'cairo-msw')
             dlls += glob.glob(os.path.join(cairo_root, arch, 'bin', '*.dll'))
 
-        # For Python 3.5 and 3.6 builds we also need to copy some VC14 redist DLLs
-        if PYVER in ['3.5', '3.6']:
+        # For Python 3.5 and 3.6 builds we also need to copy some VC14 redist DLLs.
+        # NOTE: Do it for 3.7+ too for now. But when we fully switch over to VS 2017
+        # this may need to change. See notes in wscript about it.
+        if PYVER in ['3.5', '3.6', '3.7', '3.8']:
             redist_dir = os.path.join(
                 phoenixDir(), 'packaging', 'Py3.5', 'vcredist',
                 arch, 'Microsoft.VC140.CRT', '*.dll')
@@ -1407,12 +1560,6 @@ def copyWxDlls(options):
         # for the wxlib's in $ORIGIN, so there is nothing else to do here
 
 
-# just an alias for build_py now
-def cmd_waf_py(options, args):
-    cmdTimer = CommandTimer('waf_py')
-    cmd_build_py(options, args)
-
-
 def cmd_build_py(options, args):
     cmdTimer = CommandTimer('build_py')
     waf = getWafCmd()
@@ -1424,13 +1571,15 @@ def cmd_build_py(options, args):
         os.environ['WXPYTHON_RELEASE'] = 'yes'
 
     if not isWindows:
-        WX_CONFIG = posixjoin(BUILD_DIR, 'wx-config')
-        if options.use_syswx:
-            wxcfg = posixjoin(options.prefix, 'bin', 'wx-config')
-            if options.prefix and os.path.exists(wxcfg):
-                WX_CONFIG = wxcfg
-            else:
-                WX_CONFIG = 'wx-config' # hope it is on the PATH
+        WX_CONFIG = os.environ.get('WX_CONFIG', None)
+        if WX_CONFIG is None:
+            WX_CONFIG = posixjoin(BUILD_DIR, 'wx-config')
+            if options.use_syswx:
+                wxcfg = posixjoin(options.prefix, 'bin', 'wx-config')
+                if options.prefix and os.path.exists(wxcfg):
+                    WX_CONFIG = wxcfg
+                else:
+                    WX_CONFIG = 'wx-config' # hope it is on the PATH
 
 
     wafBuildBase = wafBuildDir = getWafBuildBase()
@@ -1478,10 +1627,27 @@ def cmd_build_py(options, args):
         # folder as the wxPython extension modules.
         os.environ['LD_RUN_PATH'] = '$ORIGIN'
 
+    # Regenerate the _sysconfigdata module?
+    if options.regenerate_sysconfig:
+        do_regenerate_sysconfig()
+
     # Run waf to perform the builds
     pwd = pushDir(phoenixDir())
     cmd = '"%s" %s %s configure build %s' % (PYTHON, waf, ' '.join(build_options), options.extra_waf)
-    runcmd(cmd)
+
+    def _onWafError():
+        if options.dump_waf_log:
+            logfilename = opj(wafBuildDir, 'config.log')
+            if not os.path.exists(logfilename):
+                msg('WARNING: waf log "{}" not found!'.format(logfilename))
+                return
+
+            msg('*-'*40)
+            msg('WAF config log "{}":'.format(logfilename))
+            with open(logfilename, 'r') as log:
+                msg(log.read())
+            msg('*-'*40)
+    runcmd(cmd, onError=_onWafError)
 
     if isWindows and options.both:
         build_options.remove('--debug')
@@ -1492,6 +1658,7 @@ def cmd_build_py(options, args):
         runcmd(cmd)
 
     copyWxDlls(options)
+    cmd_build_others(options, args)
 
     cfg = Config()
     cfg.build_locale_dir(opj(cfg.PKGDIR, 'locale'))
@@ -1504,37 +1671,25 @@ def cmd_build_py(options, args):
     print("")
 
 
-def cmd_build_vagrant(options, args):
-    # Does a build and bdist_wheel for each of the Vagrant based VMs defined
-    # in {phoenixDir}/vagrant. Requires Vagrant and VirtualBox to be installed
-    # and ready for use.  Also requires a source tarball to be present in
-    # {phoenixDir}/dist, such as what is produced with cmd_sdist.
-    cmdTimer = CommandTimer('bdist_vagrant')
+def cmd_build_docker(options, args):
+    # Uses docker images defined by the Dockerfiles under ./docker/build for
+    # building wxPython wheels for a few versions of Linux.
+    #
+    # Requirements: Docker
+    #               Internet connection for downloading docker images, if needed
+    #               One wxPython source archive in ./dist (created with cmd_sdist)
+    cmdTimer = CommandTimer('build_docker')
     cfg = Config(noWxConfig=True)
-    if not options.vagrant_vms or options.vagrant_vms == 'all':
-        VMs = [ 'centos-7     all all',
-                'debian-8     all all',
-                'debian-9     all all',
-                'fedora-23    all all',
-                'fedora-26    all all',
-                'fedora-27    all gtk3', # no webkitgtk for gtk2??
-                'ubuntu-14.04 all all',
-                'ubuntu-16.04 all all',
-                'ubuntu-18.04 all all',
-                ]
-    elif options.vagrant_vms == 'none':
-        VMs = [] # to skip building anything and just upload
-    else:
-        VMs = options.vagrant_vms.split(',')
+    cmd = ['inv', 'build-wxpython']
+    if options.docker_img != 'all':
+        for img in options.docker_img.split(','):
+            cmd.append('-i')
+            cmd.append(img)
 
-    for vmName in VMs:
-        vmDir = opj(phoenixDir(), 'vagrant', vmName.split()[0])
-        pwd = pushDir(vmDir)
-        msg('Starting Vagrant VM in {}'.format(vmDir))
-        runcmd('vagrant up')
-        runcmd('vagrant ssh -c "scripts/build.sh %s"' % vmName)
-        runcmd('vagrant halt')
-        del pwd
+    # 'none' can be used to skip building and go straight to uploading
+    if options.docker_img != 'none':
+        pwd = pushDir('docker')
+        runcmd(cmd, echoCmd=True)
 
     if options.upload:
         for tag in ['gtk2', 'gtk3']:
@@ -1543,6 +1698,36 @@ def cmd_build_vagrant(options, args):
                 uploadTree(src, 'linux', options)
 
 
+def cmd_build_others(options, args):
+    # Build other stuff that may have their own seprarate build commands instead
+    # of the (ab)normal etg/tweak/generate/sip/compile sequence that the rest of
+    # wxPython uses. So far, it's just the wx.svg package
+    cmdTimer = CommandTimer('build_others')
+
+    cmd = [PYTHON, 'setup-wxsvg.py', 'build_ext', '--inplace']
+    if options.verbose:
+        cmd.append('--verbose')
+    runcmd(cmd)
+
+
+def cmd_touch_others(options, args):
+    cmdTimer = CommandTimer('touch_others')
+    pwd = pushDir(phoenixDir())
+    cfg = Config(noWxConfig=True)
+    pth = pathlib.Path(opj(cfg.PKGDIR, 'svg'))
+    for item in pth.glob('*.pyx'):
+        item.touch()
+
+
+def cmd_clean_others(options, args):
+    cmdTimer = CommandTimer('clean_others')
+    pwd = pushDir(phoenixDir())
+    cfg = Config(noWxConfig=True)
+    files = []
+    for wc in ['*.pyd', '*.so']:
+        files += glob.glob(opj(cfg.PKGDIR, 'svg', wc))
+    delFiles(files)
+
 
 def cmd_install(options, args):
     cmdTimer = CommandTimer('install')
@@ -1710,12 +1895,22 @@ def cmd_clean_py(options, args):
             files += glob.glob(opj(cfg.PKGDIR, wc))
     delFiles(files)
 
+    # Also remove any remaining DLLs just to make sure. This includes the C++
+    # runtime DLLs, Cairo, etc.
+    # TODO: Check for specific files, not just *.dll
+    if isWindows:
+        files = glob.glob(opj(cfg.PKGDIR, '*.dll'))
+        delFiles(files)
+
+
     if options.both:
         options.debug = False
         options.both = False
         cmd_clean_py(options, args)
         options.both = True
 
+    cmd_clean_others(options, args)
+
 
 def cmd_clean_sphinx(options, args):
     cmdTimer = CommandTimer('clean_sphinx')
@@ -1750,8 +1945,8 @@ def cmd_clean(options, args):
     cmd_clean_py(options, args)
 
 
-def cmd_clean_vagrant(options, args):
-    cmdTimer = CommandTimer('clean_vagrant')
+def cmd_clean_docker(options, args):
+    cmdTimer = CommandTimer('clean_docker')
     assert os.getcwd() == phoenixDir()
 
     d = opj(phoenixDir(), 'dist', 'linux')
@@ -1779,7 +1974,7 @@ def cmd_cleanall(options, args):
         files += glob.glob(wc)
     delFiles(files)
 
-    cmd_clean_vagrant(options, args)
+    cmd_clean_docker(options, args)
 
 
 def cmd_buildall(options, args):
@@ -1805,10 +2000,8 @@ def cmd_sdist(options, args):
         msg("Sorry, I don't know what to do in this source tree, no git workspace found.")
         return
 
-    # make a tree for building up the archive files
+    # Make a place to export everything to
     PDEST = 'build/sdist'
-    WSRC  = 'ext/wxWidgets'
-    WDEST = posixjoin(PDEST, WSRC)
     if not os.path.exists(PDEST):
         os.makedirs(PDEST)
 
@@ -1816,11 +2009,23 @@ def cmd_sdist(options, args):
     if not os.path.exists('dist'):
         os.mkdir('dist')
 
-    # pull out an archive copy of the repo files
-    msg('Exporting Phoenix...')
-    runcmd('git archive HEAD | tar -x -C %s' % PDEST, echoCmd=False)
-    msg('Exporting wxWidgets...')
-    runcmd('(cd %s; git archive HEAD) | tar -x -C %s' % (WSRC, WDEST), echoCmd=False)
+    # recursively export a git archive of this repo and submodules
+    def _archive_submodules(root, dest):
+        msg('Exporting {}...'.format(root))
+        if not os.path.exists(dest):
+            os.path.makedirs(dest)
+        pwd = pushDir(root)
+        runcmd('git archive HEAD | tar -x -C %s' % dest, echoCmd=False)
+
+        if os.path.exists('.gitmodules'):
+            with open('.gitmodules', 'rt') as fid:
+                for line in fid:
+                    line = line.strip()
+                    if line.startswith('path = '):
+                        sub = line[7:]
+                        _archive_submodules(sub, opj(dest, sub))
+
+    _archive_submodules('.', os.path.abspath(PDEST))
 
     # copy Phoenix's generated code into the archive tree
     msg('Copying generated files...')
@@ -1866,14 +2071,14 @@ def cmd_sdist(options, args):
     # build the tarball
     msg('Archiving Phoenix source...')
     rootname = "%s-%s" % (baseName, cfg.VERSION)
-    tarfilename = "dist/%s.tar.gz" % rootname
+    tarfilename = posixjoin(phoenixDir(), 'dist', '%s.tar.gz' % rootname)
     if os.path.exists(tarfilename):
         os.remove(tarfilename)
-    tarball = tarfile.open(name=tarfilename, mode="w:gz")
     pwd = pushDir(PDEST)
-    for name in glob.glob('*'):
-        tarball.add(name, os.path.join(rootname, name))
-    tarball.close()
+    with tarfile.open(name=tarfilename, mode="w:gz") as tarball:
+        for name in glob.glob('*'):
+            tarball.add(name, os.path.join(rootname, name), filter=_setTarItemPerms)
+
     msg('Cleaning up...')
     del pwd
     shutil.rmtree(PDEST)
@@ -1918,14 +2123,14 @@ def cmd_sdist_demo(options, args):
     # build the tarball
     msg('Archiving Phoenix demo and samples...')
     rootname = "%s-demo-%s" % (baseName, cfg.VERSION)
-    tarfilename = "dist/%s.tar.gz" % rootname
+    tarfilename = posixjoin(phoenixDir(), 'dist', '%s.tar.gz' % rootname)
     if os.path.exists(tarfilename):
         os.remove(tarfilename)
-    tarball = tarfile.open(name=tarfilename, mode="w:gz")
     pwd = pushDir(PDEST)
-    for name in glob.glob('*'):
-        tarball.add(name, os.path.join(rootname, name))
-    tarball.close()
+    with tarfile.open(name=tarfilename, mode="w:gz") as tarball:
+        for name in glob.glob('*'):
+            tarball.add(name, os.path.join(rootname, name), filter=_setTarItemPerms)
+
     msg('Cleaning up...')
     del pwd
     shutil.rmtree(PDEST)
@@ -1956,7 +2161,7 @@ def cmd_bdist(options, args):
     if isWindows and PYTHON_ARCH == '64bit':
         platform = 'win64'
     rootname = "%s-%s-%s-py%s" % (baseName, cfg.VERSION, platform, PYVER)
-    tarfilename = "dist/%s.tar.gz" % rootname
+    tarfilename = posixjoin(phoenixDir(), 'dist', '%s.tar.gz' % rootname)
 
     if not os.path.exists('dist'):
         os.makedirs('dist')
@@ -1964,23 +2169,23 @@ def cmd_bdist(options, args):
     if os.path.exists(tarfilename):
         os.remove(tarfilename)
     msg("Archiving Phoenix binaries to %s..." % tarfilename)
-    tarball = tarfile.open(name=tarfilename, mode="w:gz")
-    tarball.add('wx', opj(rootname, 'wx'),
-                filter=lambda info: None if '.svn' in info.name \
-                                            or info.name.endswith('.pyc') \
-                                            or '__pycache__' in info.name else info)
-    tarball.add(eggInfoName, opj(rootname, eggInfoName))
+    with tarfile.open(name=tarfilename, mode="w:gz") as tarball:
+        tarball.add('wx', opj(rootname, 'wx'),
+                    filter=lambda info: None if '.svn' in info.name \
+                                                or info.name.endswith('.pyc') \
+                                                or '__pycache__' in info.name else info)
+        tarball.add(eggInfoName, opj(rootname, eggInfoName))
 
-    if not isDarwin and not isWindows and not options.no_magic and not options.use_syswx:
-        # If the DLLs are not already in the wx package folder then go fetch
-        # them now.
-        msg("Archiving wxWidgets shared libraries...")
-        dlls = glob.glob(os.path.join(wxlibdir, "*%s" % dllext))
-        for dll in dlls:
-            tarball.add(dll, os.path.join(rootname, 'wx', os.path.basename(dll)))
+        if not isDarwin and not isWindows and not options.no_magic and not options.use_syswx:
+            # If the DLLs are not already in the wx package folder then go fetch
+            # them now.
+            msg("Archiving wxWidgets shared libraries...")
+            dlls = glob.glob(os.path.join(wxlibdir, "*%s" % dllext))
+            for dll in dlls:
+                tarball.add(dll, os.path.join(rootname, 'wx', os.path.basename(dll)))
+
+        tarball.add('packaging/README-bdist.txt', os.path.join(rootname, 'README.txt'))
 
-    tarball.add('packaging/README-bdist.txt', os.path.join(rootname, 'README.txt'))
-    tarball.close()
 
     if options.upload:
         uploadPackage(tarfilename, options)
@@ -2003,10 +2208,9 @@ def cmd_setrev(options, args):
 
     else:
         svnrev = getVcsRev()
-        f = open('REV.txt', 'w')
-        svnrev = '.dev'+svnrev
-        f.write(svnrev)
-        f.close()
+        with open('REV.txt', 'w') as f:
+            svnrev = '.dev'+svnrev
+            f.write(svnrev)
         msg('REV.txt set to "%s"' % svnrev)
 
     cfg = Config()
@@ -2014,6 +2218,14 @@ def cmd_setrev(options, args):
     msg('cfg.VERSION: %s' % cfg.VERSION)
 
 
+def cmd_setpythonpath(options, args):
+    cmdTimer = CommandTimer('setpythonpath')
+    assert os.getcwd() == phoenixDir()
+
+    sys.path.insert(0, phoenixDir())
+    os.environ['PYTHONPATH'] = phoenixDir()
+
+
 
 #---------------------------------------------------------------------------
 
