@@ -3,7 +3,7 @@
 # Author:      Robin Dunn
 #
 # Created:     22-Nov-2010
-# Copyright:   (c) 2010-2018 by Total Control Software
+# Copyright:   (c) 2010-2020 by Total Control Software
 # License:     wxWindows License
 #---------------------------------------------------------------------------
 
@@ -43,7 +43,7 @@ def run():
 
     etgtools.prependText(c.detailedDoc,
          "Note that it is not intended for this class to be used directly from "
-         "Python. It is wrapped just for inheriting its methods from :class:`App`.")
+         "Python. It is wrapped just for inheriting its methods in :class:`App`.")
 
     # There's no need for the command line stuff as Python has its own ways to
     # deal with that
@@ -55,7 +55,6 @@ def run():
     c.find('OnInitCmdLine').ignore()
 
     c.find('HandleEvent').ignore()
-    c.find('UsesEventLoop').ignore()
 
     # We will use OnAssertFailure, but I don't think we should let it be
     # overridden in Python.
@@ -68,6 +67,8 @@ def run():
     c.find('OnExceptionInMainLoop').ignore()
     c.find('OnFatalException').ignore()
     c.find('OnUnhandledException').ignore()
+    c.find('StoreCurrentException').ignore()
+    c.find('RethrowStoredException').ignore()
 
     # Release the GIL for potentially blocking or long-running functions
     c.find('MainLoop').releaseGIL()
@@ -122,19 +123,6 @@ def run():
                 return 0;
             #endif
             """)
-    c.addCppMethod('int', 'GetShell32Version', '()',
-        isStatic=True,
-        doc="""\
-        Returns 400, 470, 471, etc. for shell32.dll 4.00, 4.70, 4.71 or 0 if
-        it wasn't found at all.  Raises an exception on non-Windows platforms.""",
-        body="""\
-            #ifdef __WXMSW__
-                return wxApp::GetShell32Version();
-            #else
-                wxPyRaiseNotImplemented();
-                return 0;
-            #endif
-            """)
 
 
     # Remove the virtualness from these methods
@@ -236,12 +224,11 @@ def run():
     module.insertItemBefore(c, enum)
 
     module.addHeaderCode("""\
-        class wxPyApp;
-        wxPyApp* wxGetApp();
+        wxAppConsole* wxGetApp();
         """)
     module.find('wxTheApp').ignore()
     f = module.find('wxGetApp')
-    f.type = 'wxPyApp*'
+    f.type = 'wxAppConsole*'
     f.briefDoc = "Returns the current application object."
     f.detailedDoc = []
 
@@ -352,7 +339,7 @@ def run():
 
             Normally you would derive from this class and implement an ``OnInit``
             method that creates a frame and then calls ``self.SetTopWindow(frame)``,
-            however ``wx.App`` is also usable on it's own without derivation.
+            however ``wx.App`` is also usable on its own without derivation.
             """,
 
         items=[
@@ -430,7 +417,10 @@ def run():
                         self.RedirectStdio(filename)
 
                     # Use Python's install prefix as the default
-                    wx.StandardPaths.Get().SetInstallPrefix(_sys.prefix)
+                    prefix = _sys.prefix
+                    if isinstance(prefix, (bytes, bytearray)):
+                        prefix = prefix.decode(_sys.getfilesystemencoding())
+                    wx.StandardPaths.Get().SetInstallPrefix(prefix)
 
                     # Until the new native control for wxMac is up to par, still use the generic one.
                     wx.SystemOptions.SetOption("mac.listctrl.always_use_generic", 1)
@@ -447,7 +437,10 @@ def run():
                     called.  This can be overridden in derived classes, but be sure to call
                     this method from there.
                     """,
-                body="wx.StockGDI._initStockObjects()"),
+                body="""\
+                    wx.StockGDI._initStockObjects()
+                    self.InitLocale()
+                    """),
 
             PyFunctionDef('__del__', '(self)',
                 doc="",
@@ -511,6 +504,35 @@ def run():
                             self.stdioWin.size = size
                     """),
 
+            PyFunctionDef('InitLocale', '(self)',
+                doc="""\
+                    Try to ensure that the C and Python locale is in sync with wxWidgets locale.
+                    """,
+                body="""\
+                    self.ResetLocale()
+                    import locale
+                    try:
+                        loc, enc = locale.getlocale()
+                    except ValueError:
+                        loc = enc = None
+                    # Try to set it to the same language as what is already set in the C locale
+                    info = wx.Locale.FindLanguageInfo(loc) if loc else None
+                    if info:
+                        self._initial_locale = wx.Locale(info.Language)
+                    else:
+                        # otherwise fall back to the system default
+                        self._initial_locale = wx.Locale(wx.LANGUAGE_DEFAULT)
+                    """),
+
+            PyFunctionDef('ResetLocale', '(self)',
+                doc="""\
+                    Release the wx.Locale object created in :meth:`InitLocale`.
+                    This will reset the application's locale to the previous settings.
+                    """,
+                body="""\
+                    self._initial_locale = None
+                    """),
+
             PyFunctionDef('Get', '()', isStatic=True,
                 doc="""\
                     A staticmethod returning the currently active application object.
@@ -521,7 +543,6 @@ def run():
 
 
 
-
     module.addPyClass('PySimpleApp', ['App'], deprecated="Use :class:`App` instead.",
         doc="""This class is deprecated.  Please use :class:`App` instead.""",
         items=[
