@@ -5,7 +5,7 @@
 #
 # Created:     10-Sept-2011
 # Copyright:   (c) 2011 by Kevin Ollivier
-# Copyright:   (c) 2011-2018 by Total Control Software
+# Copyright:   (c) 2011-2020 by Total Control Software
 # License:     wxWindows License
 #---------------------------------------------------------------------------
 
@@ -24,6 +24,7 @@ ITEMS  = [
             'wxGraphicsBitmap',
             'wxGraphicsBrush',
             'wxGraphicsFont',
+            'wxGraphicsPenInfo',
             'wxGraphicsPen',
             'wxGraphicsContext',
             'wxGraphicsGradientStop',
@@ -60,6 +61,7 @@ def run():
     c.mustHaveApp()
     c.addCppMethod('bool', 'IsOk', '()', 'return !self->IsNull();')
     c.addCppMethod('int', '__nonzero__', '()', "return !self->IsNull();")
+    c.addCppMethod('int', '__bool__', '()', "return !self->IsNull();")
 
 
     #---------------------------------------------
@@ -73,7 +75,7 @@ def run():
         pyArgsString='(autoPaintDC) -> GraphicsContext',
         isStatic=True,
         body="""\
-            return wxGraphicsContext::Create(autoPaintDC);
+            return wxGraphicsContext::Create(*autoPaintDC);
             """)
 
     m = c.find('Create').findOverload('wxEnhMetaFileDC')
@@ -84,7 +86,7 @@ def run():
         #if wxUSE_ENH_METAFILE
             return wxGraphicsContext::Create(*metaFileDC);
         #endif
-        #endif        
+        #endif
             wxPyRaiseNotImplemented();
             return NULL;
         """)
@@ -163,7 +165,7 @@ def run():
     m = c.find('StrokeLines').findOverload('points').ignore()
     c.addCppMethod('void', 'StrokeLines', '(PyObject* points)',
         pyArgsString="(point2Ds)",
-        doc="Stroke lines conencting all the points.",
+        doc="Stroke lines connecting all the points.",
         body="""\
         size_t count;
         wxPoint2D* ptsArray = wxPoint2D_array_helper(points, &count);
@@ -189,6 +191,9 @@ def run():
         }
         """)
 
+    # TODO: support this?
+    c.find('CreateFromNativeHDC').ignore()
+
     #---------------------------------------------
     c = module.find('wxGraphicsPath')
     tools.removeVirtuals(c)
@@ -196,6 +201,11 @@ def run():
     c.find('GetCurrentPoint').findOverload('wxDouble *x, wxDouble *y').ignore()
     c.mustHaveApp()
 
+    c.find('GetNativePath').setCppCode("""\
+        if (self->IsNull())
+            return (void*)0;
+        return self->GetNativePath();
+        """)
 
     #---------------------------------------------
     c = module.find('wxGraphicsRenderer')
@@ -203,15 +213,59 @@ def run():
     markCreateFactories(c)
     c.abstract = True
 
-    for m in c.find('CreateContext').all():
-        for p in m.items:
-            if 'DC' in p.name or p.name == 'image':
-                p.keepReference = True
-    c.find('CreateContextFromImage.image').keepReference = True
 
-    # FIXME: Handle wxEnhMetaFileDC?
+    # The KeepReference annotation doesn't work for us in this case, as it will
+    # hold the reference in the renderer object, but it is better to hold the
+    # reference in the returned context object instead. Otherwise there is still
+    # some possibility that the held DC will be destroyed before the context.
+    c.addPyCode("""\
+        def _ctx_hold_ref(f):
+            from functools import wraps
+            @wraps(f)
+            def wrapper(self, obj):
+                ctx = f(self, obj)
+                if ctx is not None:
+                    ctx._obj = obj
+                return ctx
+            return wrapper
+        GraphicsRenderer.CreateContext = _ctx_hold_ref(GraphicsRenderer.CreateContext)
+        GraphicsRenderer.CreateContextFromImage = _ctx_hold_ref(GraphicsRenderer.CreateContextFromImage)
+        GraphicsRenderer.CreateContextFromUnknownDC = _ctx_hold_ref(GraphicsRenderer.CreateContextFromUnknownDC)
+        """)
+
+    # TODO: support this?
     c.find('CreateContext').findOverload('wxEnhMetaFileDC').ignore()
 
+    # TODO: support this?
+    c.find('CreateContextFromNativeHDC').ignore()
+
+    c.addPyMethod('GetType', '(self)',
+        doc="Returns the name of the GraphicsRenderer class.",
+        body="return self.GetClassInfo().GetClassName()")
+
+
+
+    c.find('GetGDIPlusRenderer').ignore()
+    c.addCppMethod('wxGraphicsRenderer*', 'GetGDIPlusRenderer', '()', isStatic=True,
+        doc="Returns GDI+ renderer (MSW only).",
+        body="""\
+            #ifdef __WXMSW__
+                return wxGraphicsRenderer::GetGDIPlusRenderer();
+            #else
+                return NULL;
+            #endif
+            """)
+
+    c.find('GetDirect2DRenderer').ignore()
+    c.addCppMethod('wxGraphicsRenderer*', 'GetDirect2DRenderer', '()', isStatic=True,
+        doc="Returns Direct2D renderer (MSW and Python3 only).",
+        body="""\
+            #if wxUSE_GRAPHICS_DIRECT2D
+                return wxGraphicsRenderer::GetDirect2DRenderer();
+            #else
+                return NULL;
+            #endif
+            """)
 
     #---------------------------------------------
     c = module.find('wxGraphicsMatrix')
@@ -234,6 +288,12 @@ def run():
     c.find('TransformPoint.x').inOut = True
     c.find('TransformPoint.y').inOut = True
 
+    c.find('GetNativeMatrix').setCppCode("""\
+        if (self->IsNull())
+            return (void*)0;
+        return self->GetNativeMatrix();
+        """)
+
 
     #---------------------------------------------
     c = module.find('wxGraphicsGradientStops')
@@ -244,6 +304,29 @@ def run():
                    factory=True)
 
 
+
+    #---------------------------------------------
+    c = module.find('wxGraphicsBitmap')
+    c.find('GetNativeBitmap').setCppCode("""\
+        if (self->IsNull())
+            return (void*)0;
+        return self->GetNativeBitmap();
+        """)
+
+
+    #---------------------------------------------
+    c = module.find('wxGraphicsPenInfo')
+    # Ignore Dashes for now
+    # TODO: we need to do something like wx.Pen.SetDashes, but since
+    # GraphicsPenInfo is transitory we can't save the reference in it to the
+    # holder, and the pen will not have been created yet...
+    c.find('Dashes').ignore()
+    c.find('GetDashes').ignore()
+    c.find('GetDashCount').ignore()
+    c.find('GetDash').ignore()
+
+
+
     #---------------------------------------------
     # Use the pyNames we set for these classes in geometry.py so the old
     # names do not show up in the docstrings, etc.
