@@ -3,7 +3,7 @@
 # Author:      Robin Dunn
 #
 # Created:     20-Dec-2012
-# Copyright:   (c) 2012-2018 by Total Control Software
+# Copyright:   (c) 2012-2020 by Total Control Software
 # License:     wxWindows License
 #---------------------------------------------------------------------------
 
@@ -19,10 +19,15 @@ DOCSTRING = ""
 # this script.
 ITEMS  = [ 'wxGridCellCoords',
 
+           'wxGridBlockCoords',
+           'wxGridBlockDiffResult',
+           'wxGridBlocks',
+
            'wxGridCellRenderer',
            'wxGridCellStringRenderer',
            'wxGridCellAutoWrapStringRenderer',
            'wxGridCellBoolRenderer',
+           'wxGridCellDateRenderer',
            'wxGridCellDateTimeRenderer',
            'wxGridCellEnumRenderer',
            'wxGridCellFloatRenderer',
@@ -30,6 +35,7 @@ ITEMS  = [ 'wxGridCellCoords',
 
            'wxGridCellEditor',
            'wxGridCellTextEditor',
+           'wxGridCellDateEditor',
            'wxGridCellAutoWrapStringEditor',
            'wxGridCellBoolEditor',
            'wxGridCellChoiceEditor',
@@ -37,6 +43,7 @@ ITEMS  = [ 'wxGridCellCoords',
            'wxGridCellFloatEditor',
            'wxGridCellNumberEditor',
 
+           'wxGridFitMode',
            'wxGridCellAttr',
 
            'wxGridCornerHeaderRenderer',
@@ -81,12 +88,22 @@ def run():
         GRID_VALUE_NUMBER =    "long"
         GRID_VALUE_FLOAT =     "double"
         GRID_VALUE_CHOICE =    "choice"
+        GRID_VALUE_DATE =      "date"
         GRID_VALUE_TEXT =      "string"
         GRID_VALUE_LONG =      "long"
         GRID_VALUE_CHOICEINT = "choiceint"
         GRID_VALUE_DATETIME =  "datetime"
         """)
 
+    # Add compatibility constants for these since they've been removed from wxWidgets
+    # TODO: Remove these in a future release.
+    module.addPyCode("""\
+        GRIDTABLE_REQUEST_VIEW_GET_VALUES = 2000
+        GRIDTABLE_REQUEST_VIEW_SEND_VALUES = 2001
+        """)
+
+    module.insertItem(0, etgtools.TypedefDef(type='wxWindow', name='wxGridWindow'))
+
     #-----------------------------------------------------------------
     c = module.find('wxGridCellCoords')
     assert isinstance(c, etgtools.ClassDef)
@@ -111,6 +128,7 @@ def run():
     c.addPyMethod('__repr__', '(self)',            'return "GridCellCoords"+str(self.Get())')
     c.addPyMethod('__len__', '(self)',             'return len(self.Get())')
     c.addPyMethod('__nonzero__', '(self)',         'return self.Get() != (0,0)')
+    c.addPyMethod('__bool__', '(self)',            'return self.Get() != (0,0)')
     c.addPyMethod('__reduce__', '(self)',          'return (GridCellCoords, self.Get())')
     c.addPyMethod('__getitem__', '(self, idx)',    'return self.Get()[idx]')
     c.addPyMethod('__setitem__', '(self, idx, val)',
@@ -126,6 +144,101 @@ def run():
                                      getItemCopy=True))
 
 
+    #-----------------------------------------------------------------
+    c = module.find('wxGridBlockCoords')
+    tools.addAutoProperties(c)
+
+    c.find('operator!').ignore()
+    c.addCppMethod('int', '__bool__', '()', "return self->operator!();")
+
+    c.addCppMethod('PyObject*', 'Get', '()', """\
+        wxPyThreadBlocker blocker;
+        return sipBuildResult(0, "(iiii)", self->GetTopRow(), self->GetLeftCol(), self->GetBottomRow(), self->GetRightCol());
+        """,
+        pyArgsString="() -> (topRow, leftCol, bottomRow, rightCol)",
+        briefDoc="Return the block coordinants as a tuple.")
+
+    c.addPyMethod('__str__', '(self)',             'return str(self.Get())')
+    c.addPyMethod('__repr__', '(self)',            'return "GridBlockCoords"+str(self.Get())')
+
+    #-----------------------------------------------------------------
+    c = module.find('wxGridBlockDiffResult')
+    c.find('m_parts').ignore()
+
+    c.addCppMethod('PyObject*', '_getParts', '()',
+        """\
+        wxPyThreadBlocker blocker;
+        PyObject* ret = PyTuple_New(4);
+        if (ret) {
+            PyTuple_SET_ITEM(ret, 0, wxPyConstructObject(&self->m_parts[0], "wxGridBlockCoords", false));
+            PyTuple_SET_ITEM(ret, 1, wxPyConstructObject(&self->m_parts[1], "wxGridBlockCoords", false));
+            PyTuple_SET_ITEM(ret, 2, wxPyConstructObject(&self->m_parts[2], "wxGridBlockCoords", false));
+            PyTuple_SET_ITEM(ret, 3, wxPyConstructObject(&self->m_parts[3], "wxGridBlockCoords", false));
+        }
+        return ret;
+        """)
+    c.addProperty('m_parts', '_getParts')
+
+
+    #-----------------------------------------------------------------
+    c = module.find('wxGridBlocks')
+    c.addPrivateDefaultCtor()
+
+    c.addPyMethod('__iter__', '(self)',
+                  'return PyGridBlocksIterator(self)',
+                  "Returns a Python iterator for acessing the collection of grid blocks.")
+
+    # This class is the Python iterator that knows how to fetch blocks from the
+    # wxGridBlocks object
+    c.addPyCode("""\
+        class PyGridBlocksIterator(object):
+            "A Python iterator for GridBlocks objects"
+            def __init__(self, blocks):
+                self._blocks = blocks
+                self._iterator = self._blocks.begin()
+
+            def __next__(self):
+                if self._iterator == self._blocks.end():
+                    raise StopIteration
+                obj = self._iterator._get()
+                self._iterator = self._iterator._next()
+                return obj
+        """)
+
+
+    # c.find('iterator').addCppMethod('wxGridBlocks::iterator', '_next', '()',
+    #                                 "return self->operator++();")
+    # c.find('iterator').addCppMethod('const wxGridBlockCoords&', '_get', '()',
+    #                                 "return self->operator*();")
+
+    # TODO: Doing these the normal way (above) breaks because it tries to use just
+    # "iterator" for the self param type, instead of wxGridBlocks::iterator.
+    # That should be fixable, but until then just add these methods the manual
+    # sip way.
+    c.find('iterator').addItem(etgtools.WigCode("""\
+        iterator& _next();
+        %MethodCode
+            PyErr_Clear();
+            Py_BEGIN_ALLOW_THREADS
+            sipRes = &sipCpp->operator++();
+            Py_END_ALLOW_THREADS
+            if (PyErr_Occurred()) return 0;
+        %End
+
+        const wxGridBlockCoords& _get() const;
+        %MethodCode
+            PyErr_Clear();
+            Py_BEGIN_ALLOW_THREADS
+            sipRes =  new ::wxGridBlockCoords(sipCpp->operator*());
+            Py_END_ALLOW_THREADS
+            if (PyErr_Occurred()) return 0;
+        %End
+
+        bool __eq__(const iterator& it) const;
+        bool __ne__(const iterator& it) const;
+        """))
+
+
     #-----------------------------------------------------------------
     c = module.find('wxGridSizesInfo')
     c.find('m_customSizes').ignore()   # TODO: Add support for wxUnsignedToIntHashMap??
@@ -237,14 +350,19 @@ def run():
                 result = sipCallMethod(0, sipMethod, "iiDN", row, col,
                                        const_cast<wxGrid *>(grid),sipType_wxGrid,NULL,
                                        new wxString(oldval),sipType_wxString,NULL);
-                if (result == Py_None) {
+                if (result == NULL) {
+                    if (PyErr_Occurred())
+                        PyErr_Print();
+                    sipRes = false;
+                }
+                else if (result == Py_None) {
                     sipRes = false;
                 }
                 else {
                     sipRes = true;
                     *newval = Py2wxString(result);
                 }
-                Py_DECREF(result);
+                Py_XDECREF(result);
                 """  if pureVirtual else "",  # only used with the base class
             )
 
@@ -281,6 +399,13 @@ def run():
     c.find('SetEditor.editor').transfer = True  # these are probably redundant now...
     c.find('SetRenderer.renderer').transfer = True
 
+    #-----------------------------------------------------------------
+    module.find('wxGridCellRendererPtr').piIgnored = True
+    module.find('wxGridCellEditorPtr').piIgnored = True
+    module.find('wxGridCellAttrPtr').piIgnored = True
+
+    module.addHeaderCode('#define sipName_ptr "ptr"')
+    module.addHeaderCode('#define sipName_tocopy "tocopy"')
 
     #-----------------------------------------------------------------
     # The instanceCode attribute is code that is used to make a default
@@ -331,7 +456,12 @@ def run():
     m.virtualCatcherCode = """\
         // virtualCatcherCode for GridTableBase.GetValue
         PyObject *result = sipCallMethod(&sipIsErr, sipMethod, "ii", row, col);
-        if (result == Py_None) {
+        if (result == NULL) {
+            if (PyErr_Occurred())
+                PyErr_Print();
+            sipRes = "";
+        }
+        else if (result == Py_None) {
             sipRes = "";
         }
         else {
@@ -395,6 +525,10 @@ def run():
     c.find('SetValueAsCustom').ignore()
 
 
+    #-----------------------------------------------------------------
+    c = module.find('wxGridStringTable')
+    c.addPrivateCopyCtor()
+
 
     #-----------------------------------------------------------------
     c = module.find('wxGridTableMessage')
@@ -404,7 +538,7 @@ def run():
     #-----------------------------------------------------------------
     c = module.find('wxGrid')
     tools.fixWindowClass(c, ignoreProtected=False)
-    c.bases = ['wxScrolledWindow']
+    c.bases = ['wxScrolledCanvas']
 
     c.find('GetColLabelAlignment.horiz').out = True
     c.find('GetColLabelAlignment.vert').out = True
@@ -439,8 +573,8 @@ def run():
     # object to the grid, so we need to optionally update the
     # ownership of the Python proxy object to match.
     c.find('SetTable').pyName = '_SetTable'
-    c.addPyMethod('SetTable', '(self, table, takeOwnership=False, selmode=Grid.SelectCells)',
-        piArgsString='(self, table, takeOwnership=False, selmode=SelectCells)',
+    c.addPyMethod('SetTable', '(self, table, takeOwnership=False, selmode=Grid.GridSelectCells)',
+        piArgsString='(self, table, takeOwnership=False, selmode=GridSelectCells)',
         doc="Set the Grid Table to be used by this grid.",
         body="""\
             val = self._SetTable(table, takeOwnership, selmode)
@@ -460,20 +594,20 @@ def run():
         """, protection='private'))
 
 
-    # Rename some nested enum values
-    c.find('wxGridSelectionModes.wxGridSelectCells').pyName = 'SelectCells'
-    c.find('wxGridSelectionModes.wxGridSelectRows').pyName = 'SelectRows'
-    c.find('wxGridSelectionModes.wxGridSelectColumns').pyName = 'SelectColumns'
-    c.find('wxGridSelectionModes.wxGridSelectRowsOrColumns').pyName = 'SelectRowsOrColumns'
-
-    # But also keep aliases for the old names, just in case
+    # Add a simpler set of names for the wxGridSelectionModes enum
     c.addPyCode("""\
-        Grid.wxGridSelectCells = Grid.SelectCells
-        Grid.wxGridSelectRows = Grid.SelectRows
-        Grid.wxGridSelectColumns = Grid.SelectColumns
-        Grid.wxGridSelectRowsOrColumns = Grid.SelectRowsOrColumns
+        Grid.SelectCells = Grid.GridSelectCells
+        Grid.SelectRows = Grid.GridSelectRows
+        Grid.SelectColumns = Grid.GridSelectColumns
+        Grid.SelectRowsOrColumns = Grid.GridSelectRowsOrColumns
         """)
 
+
+    c.find('SetCellAlignment').findOverload('align').ignore()
+    c.find('SetCellTextColour').overloads = []
+
+    c.find('GetGridWindowOffset').findOverload('int &x').ignore()
+
     #-----------------------------------------------------------------
     c = module.find('wxGridUpdateLocker')
     c.addPrivateCopyCtor()
