@@ -3,7 +3,7 @@
 # Author:      Robin Dunn
 #
 # Created:     3-Nov-2010
-# Copyright:   (c) 2010-2018 by Total Control Software
+# Copyright:   (c) 2010-2020 by Total Control Software
 # License:     wxWindows License
 #---------------------------------------------------------------------------
 
@@ -26,10 +26,6 @@ from sphinxtools.utilities import findDescendants
 #---------------------------------------------------------------------------
 # These classes simply hold various bits of information about the classes,
 # methods, functions and other items in the C/C++ API being wrapped.
-#
-# NOTE: Currently very little is being done with the docstrings. They can
-# either be reprocessed later by the document generator or we can do more
-# tinkering with them here.  It just depends on decisions not yet made...
 #---------------------------------------------------------------------------
 
 class BaseDef(object):
@@ -83,10 +79,21 @@ class BaseDef(object):
                 itemid = item.get('id')
                 if itemid and itemid.startswith('deprecated'):
                     self.deprecated = True
-                    break
+                    return
 
-            if self.deprecated:
-                break
+
+    def clearDeprecated(self):
+        """
+        Remove the deprecation notice from the detailedDoc, if any, and reset
+        self.deprecated to False.
+        """
+        self.deprecated = False
+        for para in self.detailedDoc:
+            for item in para.iter():
+                itemid = item.get('id')
+                if itemid and itemid.startswith('deprecated'):
+                    self.detailedDoc.remove(para)
+                    return
 
 
     def ignore(self, val=True):
@@ -199,6 +206,7 @@ class VariableDef(BaseDef):
         self.definition = ''
         self.argsString = ''
         self.pyInt = False
+        self.noSetter = False
         self.__dict__.update(**kw)
         if element is not None:
             self.extract(element)
@@ -240,6 +248,8 @@ class MemberVarDef(VariableDef):
         super(MemberVarDef, self).__init__()
         self.isStatic = False
         self.protection = 'public'
+        self.getCode = ''
+        self.setCode = ''
         self.__dict__.update(kw)
         if element is not None:
             self.extract(element)
@@ -591,6 +601,10 @@ class MethodDef(FunctionDef):
             self.ignore()
 
 
+    def setVirtualCatcherCode(self, code):
+        """
+        """
+        self.virtualCatcherCode = code
 
 
 #---------------------------------------------------------------------------
@@ -734,8 +748,12 @@ class ClassDef(BaseDef):
         for node in element.findall('includes'):
             self.includes.append(node.text)
         for node in element.findall('templateparamlist/param'):
-            txt = node.find('type').text
-            txt = txt.replace('class ', '')
+            if node.find('declname') is not None:
+                txt = node.find('declname').text
+            else:
+                txt = node.find('type').text
+                txt = txt.replace('class ', '')
+                txt = txt.replace('typename ', '')
             self.templateParams.append(txt)
 
         for node in element.findall('innerclass'):
@@ -798,7 +816,8 @@ class ClassDef(BaseDef):
 
 
     def includeCppCode(self, filename):
-        self.addCppCode(textfile_open(filename).read())
+        with textfile_open(filename) as fid:
+            self.addCppCode(fid.read())
 
 
     def addAutoProperties(self):
@@ -1071,6 +1090,13 @@ class ClassDef(BaseDef):
         self.addItem(WigCode(text))
 
 
+    def addDefaultCtor(self, prot='protected'):
+        # add declaration of a default constructor to this class
+        wig = WigCode("""\
+{PROT}:
+    {CLASS}();""".format(CLASS=self.name, PROT=prot))
+        self.addItem(wig)
+
     def addCopyCtor(self, prot='protected'):
         # add declaration of a copy constructor to this class
         wig = WigCode("""\
@@ -1081,6 +1107,9 @@ class ClassDef(BaseDef):
     def addPrivateCopyCtor(self):
         self.addCopyCtor('private')
 
+    def addPrivateDefaultCtor(self):
+        self.addDefaultCtor('private')
+
     def addPrivateAssignOp(self):
         # add declaration of an assignment opperator to this class
         wig = WigCode("""\
@@ -1229,6 +1258,7 @@ class CppMethodDef(MethodDef):
         self.cppSignature = cppSignature
         self.virtualCatcherCode = virtualCatcherCode
         self.isCore = _globalIsCore
+        self.isSlot = False
         self.__dict__.update(kw)
 
     @staticmethod
@@ -1441,7 +1471,8 @@ class ModuleDef(BaseDef):
             self.cppCode.append(code)
 
     def includeCppCode(self, filename):
-        self.addCppCode(textfile_open(filename).read())
+        with textfile_open(filename) as fid:
+            self.addCppCode(fid.read())
 
     def addInitializerCode(self, code):
         if isinstance(code, list):
@@ -1567,12 +1598,17 @@ class ModuleDef(BaseDef):
         return pc
 
 
-    def addGlobalStr(self, name, before=None):
+    def addGlobalStr(self, name, before=None, wide=False):
         if self.findItem(name):
             self.findItem(name).ignore()
-        gv = GlobalVarDef(type='const char*', name=name)
+        if wide:
+            gv = GlobalVarDef(type='const wchar_t*', name=name)
+        else:
+            gv = GlobalVarDef(type='const char*', name=name)
         if before is None:
             self.addItem(gv)
+        elif isinstance(before, int):
+            self.insertItem(before, gv)
         else:
             self.insertItemBefore(before, gv)
         return gv
@@ -1582,7 +1618,8 @@ class ModuleDef(BaseDef):
         """
         Add a snippet of Python code from a file to the wrapper module.
         """
-        text = textfile_open(filename).read()
+        with textfile_open(filename) as fid:
+            text = fid.read()
         return self.addPyCode(
             "#" + '-=' * 38 + '\n' +
             ("# This code block was included from %s\n%s\n" % (filename, text)) +
